# -*- coding: utf-8 -*-
# ChatTrayIcon.py
import wx

class ChatTrayIcon(wx.TaskBarIcon):
    def __init__(self, frame):
        wx.TaskBarIcon.__init__(self)
        self.Space = False
        self.frame = frame
        self.SetIcon(wx.Icon('pixmaps/dmpychat16.png', wx.BITMAP_TYPE_PNG), 'Delphi Chat 6')
        self.Bind(wx.EVT_MENU, self.OnTaskBarActivate, id=1)
        self.Bind(wx.EVT_MENU, self.OnTaskBarDeactivate, id=2)
        self.Bind(wx.EVT_MENU, self.OnTaskBarClose, id=3)
        self.Bind(wx.EVT_TASKBAR_LEFT_UP, self.OnTaskBarClick)
        self.SetAnimateTimer()

    def CreatePopupMenu(self):
        menu = wx.Menu()
        miShow = wx.MenuItem(menu, 1, u"Показать", "", wx.ITEM_NORMAL)
        miShow.SetBitmap(wx.Bitmap('pixmaps/menu/show.png'))
        menu.AppendItem(miShow)
        miHide = wx.MenuItem(menu, 2, u"Скрыть", "", wx.ITEM_NORMAL)
        miHide.SetBitmap(wx.Bitmap('pixmaps/menu/hide.png'))
        menu.AppendItem(miHide)
        
        menu.AppendSeparator()
        miExit = wx.MenuItem(menu, 3, u"Выход", u"Выйти из программы", wx.ITEM_NORMAL)
        miExit.SetBitmap(wx.Bitmap('pixmaps/menu/exit.png'))
        menu.AppendItem(miExit)
        return menu

    def OnTaskBarClose(self, event):
        if not self.frame.DialogLocked:
            self.frame.quitflag = 1
            self.frame.Close()

    def OnTaskBarActivate(self, event):
        if not self.frame.IsShown():
            self.frame.Show()
            
    def ModifyIcon(self, animated):
        if animated:
            self.timer.Start(500)
        else:
            self.timer.Stop()
            self.Space = False
            self.SetIcon(wx.Icon('pixmaps/dmpychat16.png', wx.BITMAP_TYPE_PNG), 'Delphi Chat 6')
            
    def SetAnimateTimer(self):
        self.timerId = wx.NewId()
        self.timer = wx.Timer(self, self.timerId)
        wx.EVT_TIMER(self, self.timerId, self.Animate) 
        
    def Animate(self, evt):
        self.Space = not self.Space
        if self.Space:
            self.SetIcon(wx.Icon('pixmaps/space.png', wx.BITMAP_TYPE_PNG), 'Delphi Chat 6')
        else:
            self.SetIcon(wx.Icon('pixmaps/dmpychat16.png', wx.BITMAP_TYPE_PNG), 'Delphi Chat 6')
             
    def OnTaskBarDeactivate(self, event):
        if self.frame.IsShown():
            self.frame.Hide()
            
    def OnTaskBarClick(self, event):
        if self.frame.IsActive():
            self.frame.Hide()
        else:
            self.ModifyIcon(False)
            self.frame.Show()
            self.frame.Raise()
