#!/usr/bin/env python
# -*- coding: utf-8 -*-
import wx, os
import tarfile

def chop_text(dc, text, max_size):
    x, y = dc.GetTextExtent(text)
    if x <= max_size:
        return text
    last_good_length = 0
    for i in range(len(text)):
        s = text[:i] + "..."
        x, y = dc.GetTextExtent(s)
        if (x > max_size):
            break
        last_good_length = i
    ret = text[:last_good_length] + "..."
    return ret

class VListCtrl(wx.VListBox):
    def __init__(self, parent, pluginmanager):
        super(VListCtrl, self).__init__(parent)
        self.base = self.GetTextExtent("ABCDEFHXfgkj")[1]
        self.parent = parent
        self.pm = pluginmanager
        self.bmp_d = wx.Bitmap('pixmaps/plugins/disabled.png',wx.BITMAP_TYPE_PNG)
        self.bmp_e = wx.Bitmap('pixmaps/plugins/enabled.png',wx.BITMAP_TYPE_PNG)
        self.focus_item = wx.Window(self, size=(0, 0))
        self.focus_item.Bind(wx.EVT_SET_FOCUS, self.on_focus)

        self.delete_button = wx.Button(self, label=u"Удалить", size=(-1,26))
        self.delete_button.Bind(wx.EVT_BUTTON, self.on_delete_button)
        self.disable_button = wx.Button(self, label="", size=(-1,26))
        self.disable_button.Bind(wx.EVT_BUTTON, self.on_disable_button)

        self.buttons = self.delete_button, self.disable_button
        self.Bind(wx.EVT_SIZE, self.on_size)
        self.Bind(wx.EVT_LISTBOX, self.on_selected)

        self.InitPlugins(self.pm)

        self.layout_buttons()

    def InitPlugins(self, pluginmanager, selection = 0):
        self.modules = pluginmanager.plugins
        if selection < 0: selection=0
        self.SetItemCount(len(self.modules))
        if self.modules:
            self.SetSelection(selection)
            self.SetBtnEnabledLabel(selection)
            self.ScrollToLine(selection)
        else:
            for button in self.buttons:
                button.Hide()


    def SetEnabled(self,modulename,checked):
        self.modules[self.Selection].SetActive(checked)
        if checked:
            self.pm.enabledplugins.append(modulename)
        else:
            self.pm.enabledplugins.remove(modulename)

    def GetEnabled(self,item):
        return self.modules[item].modulename in self.pm.enabledplugins

    def SetBtnEnabledLabel(self,item):
        if item != -1:
            if self.GetEnabled(item):
                self.disable_button.SetLabel(label=u"Отключить")
            else:
                self.disable_button.SetLabel(label=u"Включить")

    def on_size(self, event):
        self.layout_buttons()
        event.Skip()

    def layout_buttons(self):
        self.delete_button.Position =(
            self.ClientSize[0] - self.delete_button.Size[0] - 6, -1
            )
        self.disable_button.Position =(
            self.ClientSize[0] - self.delete_button.Size[0]-self.disable_button.Size[0] - 10,
            -1
            )

    def on_delete_button(self, event):
        if self.Selection == -1:
                return
        module = self.modules[self.Selection]
        text = '"%s" [module %s.py]' % (module.name, module.modulename)
        dial = wx.MessageDialog(self, u'Сейчас буду удалять %s.\nВы готовы?' % text, u'Удаление',
            wx.YES_NO | wx.ICON_QUESTION)
        if(dial.ShowModal()==wx.ID_YES):
            if self.GetEnabled(self.Selection):
                self.SetEnabled(module.modulename,False)

            os.remove("plugins/%s.py" % module.modulename)
            self.pm.InitPlugins()
            self.InitPlugins(self.pm, self.Selection-1)
            self.RefreshAll()
        # set focus back to the panel, not to the button
        self.SetFocusIgnoringChildren()

    def on_disable_button(self, event):
        if self.Selection == -1:
                return
        self.SetEnabled(self.modules[self.Selection].modulename,not self.GetEnabled(self.Selection))
        self.SetBtnEnabledLabel(self.Selection)
        self.RefreshLine(self.Selection)
        self.SetFocusIgnoringChildren()
        #event.Skip()

    def on_focus(self, event):
        self.SetFocusIgnoringChildren()

    def OnMeasureItem(self, item):
        dh = self.GetTextExtent(self.modules[item].description)[1]
        if self.IsCurrent(item):
                return self.base * 2 + 6 + self.buttons[0].Size[1] + 12 + dh

        return self.base * 2 + 6 + 6 + dh

    def on_selected(self, event):
        event.Skip()
        self.Freeze()
        self.SetBtnEnabledLabel(event.GetSelection())
        for item in self.buttons:
                item.Hide()

        self.RefreshAll()
        self.Thaw()

    def OnDrawItem(self, dc, rect, n):
        if not self.IsVisible(self.Selection):
            for item in self.buttons:
                item.Hide()
        module = self.modules[n]
        if self.IsCurrent(n):
            for item in self.buttons:
                item.Show()
            pos = self.base * 2 + 6 + 6 + self.GetTextExtent(module.description)[1]
            if self.buttons[0].Position[-1] != rect.Y + pos:
                for button in self.buttons:
                    button.Position = -1, rect.Y + pos
            colour = wx.SystemSettings.GetColour(wx.SYS_COLOUR_HIGHLIGHTTEXT)
        else:
            colour = wx.SystemSettings.GetColour(wx.SYS_COLOUR_WINDOWTEXT)

        if self.GetEnabled(n):
            dc.DrawBitmap(self.bmp_e, rect.X+2,rect.Y+4, True)
        else:
            dc.DrawBitmap(self.bmp_d, rect.X+2,rect.Y+4, True)
        dc.SetTextForeground(colour)

        font = self.Font
        size = font.GetPointSize()
        font.SetWeight(wx.FONTWEIGHT_BOLD)
        font.SetPointSize(size+2)
        dc.Font = font

        name = module.name
        modulename = "[%s.py]" % module.modulename
        version = module.version

        w, h = dc.GetTextExtent(name)
        dc.DrawText(name, rect.X + 28, rect.Y + 6)
        nwidth = dc.GetTextExtent(name)[0]
        font.SetPointSize(size)
        font.SetWeight(wx.FONTWEIGHT_NORMAL)
        dc.Font = font

        dc.DrawText(modulename, nwidth+42, rect.Y + 9)
        if version:
            vleft = self.ClientSize[0] - dc.GetTextExtent(version)[0] - 10
            dc.DrawText(version, vleft, rect.Y + 10)

        description = chop_text(dc, module.description, rect.Width - 32)
        dc.DrawText(description,
            rect.X + 30, rect.Y + 10 + self.base + 6)

    def OnDrawBackground(self, dc, rect, n):
        if self.IsCurrent(n):
            dc.Brush = wx.Brush(
                wx.SystemSettings.GetColour(wx.SYS_COLOUR_HIGHLIGHT)
            )
            dc.Pen = wx.TRANSPARENT_PEN
            dc.DrawRectangle(*rect)
        else:
            dc.Brush = wx.Brush(
                wx.SystemSettings.GetColour(wx.SYS_COLOUR_WINDOW)
            )
            dc.Pen = wx.TRANSPARENT_PEN
            dc.DrawRectangle(*rect)

    def OnDrawSeparator(self, dc, rect, n):
        dc.Pen = wx.Pen((192, 192, 192))
        for item in range(rect.Width)[::2]:
            dc.DrawPoint(item, rect.Y + rect.Height - 1)
        rect.Height -= 1


class frmPlugins(wx.Dialog):
    def __init__(self, *args, **kwds):
        kwds["style"] = wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER|wx.THICK_FRAME
        wx.Dialog.__init__(self, *args, **kwds)
        self.parent = args[0]
        self.pm = self.parent.PluginManager

        self.vlbPlugins = VListCtrl(self, self.pm)

        self.btnAdd = wx.Button(self, wx.ID_ADD, "")
        self.btnClose = wx.Button(self, wx.ID_OK, "")

        self.__set_properties()
        self.__do_layout()

    def __set_properties(self):
        self.SetTitle("Plugins Manager")
        self.SetSize((500, 400))
        self.Bind(wx.EVT_BUTTON, self.OnClose, self.btnClose)
        self.Bind(wx.EVT_BUTTON, self.OnAdd, self.btnAdd)

    def __do_layout(self):
        sizerTop = wx.BoxSizer(wx.VERTICAL)
        sizerBottom = wx.BoxSizer(wx.HORIZONTAL)
        sizerTop.Add(self.vlbPlugins, 1, wx.EXPAND, 0)
        sizerBottom.Add(self.btnAdd, 0, wx.ALL, 7)
        sizerBottom.Add((20, 20), 1, wx.EXPAND, 0)
        sizerBottom.Add(self.btnClose, 0, wx.ALL, 7)
        sizerTop.Add(sizerBottom, 0, wx.EXPAND, 0)
        self.SetSizer(sizerTop)
        self.Layout()
        self.Centre()

    def OnClose(self,event):
        self.SaveConfig()
        event.Skip()

    def SaveConfig(self):
        self.parent.cfg.Write("enabledplugins", ','.join(self.parent.PluginManager.enabledplugins))
        self.parent.cfg.Flush()

    def IsPlugin(self, tar):
        for tarinfo in tar.getmembers():
            if "plugins" in tarinfo.name:
                return True
                break
        return False

    def OnAdd(self, event):
        wcd = 'Delphichat plugins pack (*.d6p)|*.d6p|All files (*)|*'
        dir = os.getcwd()
        open_dlg = wx.FileDialog(self, message='Choose a plugin', defaultDir=dir, defaultFile='',
            wildcard=wcd, style=wx.OPEN)
        if open_dlg.ShowModal() == wx.ID_OK:
            pluginfile = open_dlg.GetPath()
            #os.chdir(wx.GetApp().path)
            tar = tarfile.open(pluginfile)

            if not self.IsPlugin(tar):
                wx.MessageDialog(self, u'Это не похоже на плагин.', u'Ошибка',
                    wx.OK | wx.ICON_ERROR).ShowModal()
            else:
                tar.extractall()
                self.pm.InitPlugins()
                self.vlbPlugins.InitPlugins(self.pm)
                self.vlbPlugins.RefreshAll()
            tar.close()
        open_dlg.Destroy()

        event.Skip()
# end of class Plugins

